@@ -1,16 +1,16 @@
 /* gtkam-chooser.c
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
+ * Copyright  2001 Lutz Mller <lutz@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
@@ -18,27 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtkam-chooser.h"
-
-#ifdef ENABLE_NLS
-#  include <libintl.h>
-#  undef _
-#  define _(String) dgettext (PACKAGE, String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else
-#  define textdomain(String) (String)
-#  define gettext(String) (String)
-#  define dgettext(Domain,Message) (Message)
-#  define dcgettext(Domain,Message,Type) (Message)
-#  define bindtextdomain(Domain,Directory) (Domain)
-#  define _(String) (String)
-#  define N_(String) (String)
-#endif
+#include "i18n.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -321,14 +303,15 @@ on_apply_clicked (GtkButton *button, GtkamChooser *chooser)
 		return;
 
 	camera = gtkam_chooser_get_camera (chooser);
-	if (camera) {
+	if (camera && camera->camera) {
 		g_signal_emit (G_OBJECT (chooser), signals[CAMERA_SELECTED],
 			       0, camera);
 		g_object_unref (G_OBJECT (camera));
 
 		chooser->priv->needs_update = FALSE;
 		gtk_widget_set_sensitive (chooser->apply_button, FALSE);
-	}
+	} else if (camera)
+		g_object_unref (G_OBJECT (camera));
 }
 
 static void
@@ -345,12 +328,13 @@ on_ok_clicked (GtkButton *button, GtkamChooser *chooser)
 
 	if (chooser->priv->needs_update) {
 		camera = gtkam_chooser_get_camera (chooser);
-		if (camera) {
+		if (camera && camera->camera) {
 			g_signal_emit (G_OBJECT (chooser),
 				       signals[CAMERA_SELECTED], 0, camera);
 			g_object_unref (G_OBJECT (camera));
 			gtk_object_destroy (GTK_OBJECT (chooser));
-		}
+		} else if (camera)
+			g_object_unref (G_OBJECT (camera));
 	} else
 		gtk_object_destroy (GTK_OBJECT (chooser));
 }
@@ -461,7 +445,7 @@ on_detect_clicked (GtkButton *button, GtkamChooser *chooser)
 						      GTK_WINDOW (chooser));
 			gtk_widget_show (d);
 		} else {
-			//FIXME: Let user choose from the list
+			/* FIXME: Let user choose from the list */
 			gp_list_get_name (&list, 0, &name);
 			gtk_entry_set_text (chooser->priv->entry_model, name);
 			gtk_entry_set_text (chooser->priv->entry_port,
@@ -517,7 +501,7 @@ gtkam_chooser_set_port_list (GtkamChooser *chooser, GList *list)
 	g_return_if_fail (GTKAM_IS_CHOOSER (chooser));
 
         if (list) {
-		
+
 		/* Remember the old entry */
 		port = g_strdup (gtk_entry_get_text (
 						chooser->priv->entry_port));
@@ -585,7 +569,6 @@ gtkam_chooser_new (void)
 {
 	GtkamChooser *chooser;
 	GtkWidget *table, *label, *button, *combo, *vbox, *check;
-	gchar *t;
 
 	chooser = g_object_new (GTKAM_TYPE_CHOOSER, NULL);
 
@@ -599,21 +582,19 @@ gtkam_chooser_new (void)
 	gp_port_info_list_new (&(chooser->priv->il));
 	gp_port_info_list_load (chooser->priv->il);
 
-	t = g_locale_to_utf8 (_("Select Camera"), -1, NULL, NULL, NULL);
-	gtk_window_set_title (GTK_WINDOW (chooser), t);
-	g_free (t);
+	gtk_window_set_title (GTK_WINDOW (chooser), _("Add Camera"));
 	gtk_container_set_border_width (GTK_CONTAINER (chooser), 5);
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (GTKAM_DIALOG (chooser)->vbox),
-			    vbox, TRUE, TRUE, 0);
+			    vbox, FALSE, FALSE, 0);
 
 	table = gtk_table_new (3, 3, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 5);
 	gtk_widget_show (table);
-	gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
 	chooser->priv->table = table;
 
 	label = gtk_label_new (_("Model:"));
@@ -628,10 +609,7 @@ gtkam_chooser_new (void)
 #if 0
 	gtk_entry_set_editable (chooser->priv->entry_model, FALSE);
 #endif
-
-	t = g_locale_to_utf8 (_("Detect"), -1, NULL, NULL, NULL);
-	button = gtk_button_new_with_label (t);
-	g_free (t);
+	button = gtk_button_new_with_label (_("Detect"));
 	gtk_widget_show (button);
 	gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
@@ -673,34 +651,26 @@ gtkam_chooser_new (void)
 #if 0
 	gtk_entry_set_editable (chooser->priv->entry_speed, FALSE);
 #endif
-
-	t = g_locale_to_utf8 (_("Allow multiple frontends"), -1,
-			      NULL, NULL, NULL);
-	check = gtk_check_button_new_with_label (t);
-	g_free (t);
+	check = gtk_check_button_new_with_label (_("Allow multiple frontends"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), FALSE);
 	gtk_table_attach_defaults (GTK_TABLE (table), check, 0, 2, 3, 4);
 	g_signal_connect (GTK_OBJECT (check), "toggled",
 			    GTK_SIGNAL_FUNC (on_multi_toggled), chooser);
 	chooser->priv->check_multi = check;
 
-	t = g_locale_to_utf8 (_("Enhanced"), -1, NULL, NULL, NULL);
-	button = gtk_toggle_button_new_with_label (t);
-	g_free (t);
+	button = gtk_toggle_button_new_with_label (_("Enhanced"));
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (chooser)->action_area),
 			   button);
 	g_signal_connect (GTK_OBJECT (button), "toggled",
 			    GTK_SIGNAL_FUNC (on_more_options_toggled), chooser);
 
-	button = gtk_button_new_from_stock (GTK_STOCK_OK);
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (chooser)->action_area),
 			   button);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC (on_ok_clicked), chooser);
-	gtk_widget_grab_focus (button);
-	chooser->priv->ok = button;
+			    GTK_SIGNAL_FUNC (on_cancel_clicked), chooser);
 
 	chooser->apply_button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
 	gtk_widget_show (chooser->apply_button);
@@ -710,12 +680,14 @@ gtkam_chooser_new (void)
 	g_signal_connect (GTK_OBJECT (chooser->apply_button), "clicked",
 			    GTK_SIGNAL_FUNC (on_apply_clicked), chooser);
 
-	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	button = gtk_button_new_from_stock (GTK_STOCK_OK);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (chooser)->action_area),
 			   button);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC (on_cancel_clicked), chooser);
+			    GTK_SIGNAL_FUNC (on_ok_clicked), chooser);
+	gtk_widget_grab_focus (button);
+	chooser->priv->ok = button;
 
 	/* Fill the model combo with all models */
 	gtkam_chooser_set_camera_mask (chooser, GP_OPERATION_NONE);
