@@ -1,6 +1,6 @@
 /* gtkam-delete.c
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
+ * Copyright  2001 Lutz Mller <lutz@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,27 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtkam-delete.h"
-
-#ifdef ENABLE_NLS
-#  include <libintl.h>
-#  undef _
-#  define _(String) dgettext (PACKAGE, String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else
-#  define textdomain(String) (String)
-#  define gettext(String) (String)
-#  define dgettext(Domain,Message) (Message)
-#  define dcgettext(Domain,Message,Type) (Message)
-#  define bindtextdomain(Domain,Directory) (Domain)
-#  define _(String) (String)
-#  define N_(String) (String)
-#endif
+#include "i18n.h"
 
 #include <string.h>
 
@@ -346,13 +328,6 @@ gtkam_delete_new (void)
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled),
 					       delete->priv->vbox);
 
-	button = gtk_button_new_from_stock (GTK_STOCK_DELETE);
-	gtk_widget_show (button);
-	g_signal_connect (GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC (on_delete_clicked), delete);
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (delete)->action_area),
-			   button);
-
 	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
 	gtk_widget_show (button);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
@@ -361,6 +336,13 @@ gtkam_delete_new (void)
 			   button);
 	gtk_widget_grab_focus (button);
 
+	button = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+	gtk_widget_show (button);
+	g_signal_connect (GTK_OBJECT (button), "clicked",
+			    GTK_SIGNAL_FUNC (on_delete_clicked), delete);
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (delete)->action_area),
+			   button);
+
 	return (GTK_WIDGET (delete));
 }
 
