@@ -1,6 +1,6 @@
 /* gtkam-save.c
  *
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
+ * Copyright  2001 Lutz Mller <lutz@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,30 +18,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtkam-save.h"
-
-#ifdef ENABLE_NLS
-#  include <libintl.h>
-#  undef _
-#  define _(String) dgettext (PACKAGE, String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else
-#  define textdomain(String) (String)
-#  define gettext(String) (String)
-#  define dgettext(Domain,Message) (Message)
-#  define dcgettext(Domain,Message,Type) (Message)
-#  define bindtextdomain(Domain,Directory) (Domain)
-#  define _(String) (String)
-#  define N_(String) (String)
-#endif
+#include "i18n.h"
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkhbox.h>
@@ -85,6 +68,7 @@ struct _GtkamSavePrivate
 	gboolean quiet, err_shown;
 
 	GtkWidget *status;
+	GtkWindow *main_window;
 };
 
 #define PARENT_TYPE GTK_TYPE_FILE_SELECTION
@@ -172,6 +156,33 @@ on_cancel_clicked (GtkButton *button, GtkamSave *save)
 	gtk_widget_destroy (GTK_WIDGET (save));
 }
 
+static int
+default_numbering_start(const GtkamSave *save)
+{
+	GDir *dir;
+	const gchar *file;
+	gchar *tail,*prefix;
+	int max=1;
+	int plen,current;
+
+	if (!(dir=g_dir_open(gtk_file_selection_get_filename(
+		GTK_FILE_SELECTION(save)),0,NULL))) return 1;
+	prefix=g_strdup(gtk_entry_get_text(GTK_ENTRY(save->priv->prefix_entry)));
+	plen=strlen(prefix);
+	while ((file=g_dir_read_name(dir))) {
+		/* compare prefix */
+		if (strncmp(prefix,file,plen)) continue;
+		/* check number */
+		current=strtol(file+plen,&tail,10);
+		if (tail - (file+plen) != 4) continue;
+		/* FIXME: check for regular file here */
+		if (max <= current) max = current+1;
+	}
+	g_free(prefix);
+	g_dir_close(dir);
+	return max;
+}
+
 static void
 on_filename_camera_toggled (GtkToggleButton *toggle, GtkamSave *save)
 {
@@ -212,6 +223,12 @@ on_filename_camera_toggled (GtkToggleButton *toggle, GtkamSave *save)
 			!toggle->active);
 		gtk_widget_set_sensitive (GTK_WIDGET (save->priv->spin_entry),
 			!toggle->active);
+		if (!toggle->active) {
+			/* calculate default start number */
+			gtk_spin_button_set_value (
+				GTK_SPIN_BUTTON (save->priv->spin_entry),
+				default_numbering_start(save));
+		}
 
 		gtk_widget_hide (GTK_FILE_SELECTION (save)->file_list);
 		gtk_widget_set_sensitive (GTK_FILE_SELECTION (save)->file_list,
@@ -240,6 +257,9 @@ create_full_filename (const gchar *filename, CameraFileType type)
 	case GP_FILE_TYPE_AUDIO:
 		full_filename = g_strdup_printf ("audio_%s", filename);
 		break;
+	case GP_FILE_TYPE_EXIF:
+		full_filename = g_strdup_printf ("exif_%s", filename);
+		break;
 	default:
 		full_filename = g_strdup (filename);
 		break;
@@ -259,11 +279,10 @@ concat_dir_and_file (const gchar *dirname, const gchar *filename)
 		full_path = g_strdup_printf ("%s%s", dirname, filename);
 	else
 		full_path = g_strdup_printf ("%s/%s", dirname, filename);
-
 	return (full_path);
 }
 
-static void
+static int
 save_file (GtkamSave *save, CameraFile *file, guint n)
 {
 	gchar *full_path, *full_filename, *dirname, *msg, *number_filename;
@@ -283,27 +302,18 @@ save_file (GtkamSave *save, CameraFile *file, guint n)
 
 		/* Use filename provided by the CameraFile */
 		full_filename = create_full_filename (filename, type);
-
-		if (g_slist_length (save->priv->data) > 1) {
-			dirname = g_dirname (fsel_path);
-			full_path = concat_dir_and_file (dirname,
-							 full_filename);
-			g_free (dirname);
-		} else {
-			full_path = concat_dir_and_file (fsel_path,
-							 full_filename);
-		}
+		full_path = concat_dir_and_file (fsel_path, full_filename);
 		g_free (full_filename);
 
 	} else {
 		if (g_slist_length (save->priv->data) == 1) {
 
 			/* Use filename provided by the GtkFileSelection */
-			fsel_filename = g_basename (fsel_path);
+			fsel_filename = g_path_get_basename (fsel_path);
 
 			full_filename = create_full_filename (fsel_filename,
 							      type);
-			dirname = g_dirname (fsel_path);
+			dirname = g_path_get_dirname (fsel_path);
 			full_path = concat_dir_and_file (dirname,
 							 full_filename);
 			g_free (dirname);
@@ -314,8 +324,7 @@ save_file (GtkamSave *save, CameraFile *file, guint n)
 			/* Use filename in prefix */
 			prefix = gtk_entry_get_text (
 					GTK_ENTRY (save->priv->prefix_entry));
-			if (!prefix)
-				prefix = "photo";
+			if (!g_strcasecmp (prefix, "")) prefix = _("photo");
 
 			suffix = strrchr (mime_type, '/');
 			suffix++;
@@ -332,34 +341,56 @@ save_file (GtkamSave *save, CameraFile *file, guint n)
 
 	/* Check for existing file */
 	if (!save->priv->quiet && file_exists (full_path)) {
-		msg = g_strdup_printf (_("'%s' already exists."),
+		msg = g_strdup_printf (_("The file '%s' already exists."),
 				       full_path);
-		dialog = gtkam_close_new (msg);
-		gtk_window_set_transient_for (GTK_WINDOW (dialog),
-					      GTK_WINDOW (save));
-		gtk_widget_show (dialog);
-		g_free (msg);
-		g_free (full_path);
-		return;
+		if (!save->priv->err_shown) {
+			dialog = gtkam_error_new (GP_ERROR_FILE_EXISTS, NULL, GTK_WIDGET (save), msg);
+			gtk_window_set_transient_for (GTK_WINDOW (dialog),
+						      save->priv->main_window);
+			gtk_widget_show (dialog);
+			g_free (msg);
+			g_free (full_path);
+			save->priv->err_shown = TRUE;
+		}
+		return 0;
 	}
 
+	/* FIXME Check for sufficient disk space for this file, or
+	   calculate total disk space required for all files before
+	   save process starts */
+	
 	result = gp_file_save (file, full_path);
 	if (result < 0) {
-		dialog = gtkam_error_new (result, NULL, GTK_WIDGET (save),
-				_("Could not save file to '%s'."), full_path);
-		gtk_widget_show (dialog);
+		if (!save->priv->err_shown) {
+			dialog = gtkam_error_new (result, NULL, GTK_WIDGET (save),
+					_("Could not save file to '%s'."), full_path);
+			gtk_widget_show (dialog);
+			save->priv->err_shown = TRUE;
+		}
 	} else {
 		progname = gtk_entry_get_text (save->priv->program);
-		if (progname && fork ()) {
+		if (progname && progname[0] != '\0' && fork ()) {
+			/*
+			 * The parent process execs. This means the child
+			 * becomes the gtkam, and the parent exits after
+			 * the viewer runs. 
+			 *
+			 * If you specify a viewer, the originating gtkam
+			 * appears to complete. This should really leave
+			 * the parent, and have the child exec, but then
+			 * we need a child reaper based on signal
+			 * handling.
+			 */
 			execlp (progname, progname, full_path, NULL);
 			_exit (0);
 		} 
 	}
 
 	g_free (full_path);
+	return result;
 }
 
-static void
+static int
 get_file (GtkamSave *save, GtkamCamera *camera,
 	  const gchar *folder, const gchar *name, CameraFileType type, guint n,
 	  GtkamContext *context)
@@ -375,7 +406,7 @@ get_file (GtkamSave *save, GtkamCamera *camera,
 		gp_camera_exit (camera->camera, NULL);
 	switch (result) {
 	case GP_OK:
-		save_file (save, file, n);
+		result = save_file (save, file, n);
 		break;
 	case GP_ERROR_CANCEL:
 		break;
@@ -390,24 +421,100 @@ get_file (GtkamSave *save, GtkamCamera *camera,
 		}
 	}
 	gp_file_unref (file);
+	return result;
 }
 
+static void
+store_save_settings(GtkamSave *save)
+{
+	gchar *savedir,*t;
+	gchar buf[5];
+
+	/* directory */
+	savedir=g_strdup(gtk_file_selection_get_filename(
+			      GTK_FILE_SELECTION (save)));
+	if (savedir) {
+		if (strlen(savedir) > 255) savedir[255]='\0';
+		if (!g_file_test(savedir,G_FILE_TEST_IS_DIR)) {
+			t=g_path_get_dirname(savedir);
+			g_free(savedir);
+			savedir=t;
+		}
+		gp_setting_set("gtkam","save-dir",savedir);
+		g_free(savedir);
+	}
+	/* toggle buttons */
+#define STORE_TOGGLE(NAME) \
+	snprintf(buf,sizeof(buf), "%d", gtk_toggle_button_get_active(	\
+		save->priv->toggle_ ## NAME)); 				\
+	gp_setting_set("gtkam","save-" #NAME,buf)
+	
+	STORE_TOGGLE(preview);
+	STORE_TOGGLE(normal);
+	STORE_TOGGLE(raw);
+	STORE_TOGGLE(audio);
+	STORE_TOGGLE(exif);
+	STORE_TOGGLE(filename_camera);
+	/* prefix entry */
+	if (g_slist_length (save->priv->data) > 1) {
+		/* only when saving several files */
+		gp_setting_set("gtkam","save-prefix",(gchar *)
+			gtk_entry_get_text(GTK_ENTRY(save->priv->prefix_entry)));
+	}
+}
+
+static void
+load_save_settings(GtkamSave *save)
+{
+	gchar buf[1024];
+	gchar *dir;
+
+	/* directory */
+	if (gp_setting_get("gtkam","save-dir",buf) == GP_OK) {
+		dir=g_strdup(buf);
+		strcat(buf,G_DIR_SEPARATOR_S);
+		gtk_file_selection_set_filename(GTK_FILE_SELECTION(save),buf);
+	} else {
+		dir=g_strdup(gtk_file_selection_get_filename(
+			GTK_FILE_SELECTION(save) ));
+	}
+	/* prefix entry */
+	gp_setting_get("gtkam","save-prefix",buf);
+	gtk_entry_set_text(GTK_ENTRY(save->priv->prefix_entry), buf);
+
+	/* toggle buttons */
+#define LOAD_TOGGLE(NAME) \
+	gp_setting_get("gtkam","save-" #NAME,buf); 			\
+	gtk_toggle_button_set_active(save->priv->toggle_ ## NAME, atoi(buf))
+
+	LOAD_TOGGLE(preview);
+	LOAD_TOGGLE(normal);
+	LOAD_TOGGLE(raw);
+	LOAD_TOGGLE(audio);
+	LOAD_TOGGLE(exif);
+	LOAD_TOGGLE(filename_camera);
+	g_free(dir);
+}
+
+
 static void
 on_ok_clicked (GtkButton *button, GtkamSave *save)
 {
 	guint i, count, j = 1;
-	GtkWidget *s;
+	int result = 0;
+	GtkWidget *s, *dialog;
 	unsigned int id = 0;
 	GtkamSaveData *data;
 
+	store_save_settings(save);
 	gtk_widget_hide (GTK_WIDGET (save));
 
 	count = g_slist_length (save->priv->data);
 	if (count == 1)
-		s = gtkam_cancel_new (_("Downloading file..."));
+		s = gtkam_cancel_new (_("Downloading file"));
 	else
-		s = gtkam_cancel_new (_("Downloading %i files..."), count);
-	gtk_window_set_transient_for (GTK_WINDOW (s), GTK_WINDOW (save));
+		s = gtkam_cancel_new (_("Downloading %i files"), count);
+	gtk_window_set_transient_for (GTK_WINDOW (s), save->priv->main_window);
 	gtk_widget_show (s);
 
 	if (count > 1)
@@ -428,32 +535,53 @@ on_ok_clicked (GtkButton *button, GtkamSave *save)
 
 		if (save->priv->toggle_normal &&
 		    save->priv->toggle_normal->active)
-			get_file (save, data->camera,
+			result = get_file (save, data->camera,
 				  data->folder, data->name,
 				  GP_FILE_TYPE_NORMAL, i + j,
 				  GTKAM_CANCEL (s)->context);
 		if (save->priv->toggle_preview &&
-		    save->priv->toggle_preview->active)
-			get_file (save, data->camera,
+		    save->priv->toggle_preview->active &&
+		    (!result < 0))
+			result = get_file (save, data->camera,
 				  data->folder, data->name,
 				  GP_FILE_TYPE_PREVIEW, i + j,
 				  GTKAM_CANCEL (s)->context);
 		if (save->priv->toggle_raw &&
-		    save->priv->toggle_raw->active)
-			get_file (save, data->camera,
+		    save->priv->toggle_raw->active &&
+		    (!result < 0))
+			result = get_file (save, data->camera,
 				  data->folder, data->name, GP_FILE_TYPE_RAW,
 				  i + j, GTKAM_CANCEL (s)->context);
 		if (save->priv->toggle_audio &&
-		    save->priv->toggle_audio->active)
-			get_file (save, data->camera,
+		    save->priv->toggle_audio->active &&
+		    (!result < 0))
+			result = get_file (save, data->camera,
 				  data->folder, data->name, GP_FILE_TYPE_AUDIO,
 				  i + j, GTKAM_CANCEL (s)->context);
 		if (save->priv->toggle_exif &&
-		    save->priv->toggle_exif->active)
-			get_file (save, data->camera,
+		    save->priv->toggle_exif->active &&
+		    (!result < 0))
+			result = get_file (save, data->camera,
 				  data->folder, data->name, GP_FILE_TYPE_EXIF,
 				  i + j, GTKAM_CANCEL (s)->context);
 
+		if (result < 0) {
+			if (count > 1)
+				gp_context_progress_stop (GTKAM_CANCEL (s)->context->context, id);
+			if (!save->priv->err_shown) {
+				dialog = gtkam_error_new (result, GTKAM_CANCEL (s)->context,
+						GTK_WIDGET (save), _("Problem getting '%s' "
+						"from folder '%s'."),
+						data->name, data->folder);
+				gtk_widget_show (dialog);
+				save->priv->err_shown = TRUE;
+			}
+	
+			gtk_object_destroy (GTK_OBJECT (s));
+			gtk_object_destroy (GTK_OBJECT (save));
+			return;
+		}
+		
 		if (count > 1)
 			gp_context_progress_update (
 				GTKAM_CANCEL (s)->context->context, id, i + 1);
@@ -466,18 +594,16 @@ on_ok_clicked (GtkButton *button, GtkamSave *save)
 		gp_context_progress_stop (
 				GTKAM_CANCEL (s)->context->context, id);
 	gtk_object_destroy (GTK_OBJECT (s));
-
 	gtk_object_destroy (GTK_OBJECT (save));
 }
 
 GtkWidget *
-gtkam_save_new (void)
+gtkam_save_new (GtkWindow *main_window)
 {
 	GtkamSave *save;
 	GtkWidget *hbox, *frame, *check, *label, *entry;
 	GtkObject *a;
 	GtkTooltips *tooltips;
-	gchar *t;
 
 	save = g_object_new (GTKAM_TYPE_SAVE, NULL);
 
@@ -497,9 +623,7 @@ gtkam_save_new (void)
 	gtk_widget_show (hbox);
 	gtk_container_add (GTK_CONTAINER (frame), hbox);
 
-	t = g_locale_to_utf8 (_("Save photos"), -1, NULL, NULL, NULL);
-	check = gtk_check_button_new_with_label (t);
-	g_free (t);
+	check = gtk_check_button_new_with_label (_("Save photos"));
 	gtk_widget_show (check);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
 	gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, TRUE, 0);
@@ -507,36 +631,28 @@ gtkam_save_new (void)
 			      "this is checked"), NULL);
 	save->priv->toggle_normal = GTK_TOGGLE_BUTTON (check);
 
-	t = g_locale_to_utf8 (_("Save raw data"), -1, NULL, NULL, NULL);
-	check = gtk_check_button_new_with_label (t);
-	g_free (t);
+	check = gtk_check_button_new_with_label (_("Save raw data"));
 	gtk_widget_show (check);
 	gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, TRUE, 0);
 	gtk_tooltips_set_tip (tooltips, check, _("Raw data will be "
 			      "saved if this is checked"), NULL);
 	save->priv->toggle_raw = GTK_TOGGLE_BUTTON (check);
 
-	t = g_locale_to_utf8 (_("Save audio data"), -1, NULL, NULL, NULL);
-	check = gtk_check_button_new_with_label (t);
-	g_free (t);
+	check = gtk_check_button_new_with_label (_("Save audio data"));
 	gtk_widget_show (check);
 	gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, TRUE, 0);
 	gtk_tooltips_set_tip (tooltips, check, _("Audio data will be "
 			      "saved if this is checked"), NULL);
 	save->priv->toggle_audio = GTK_TOGGLE_BUTTON (check);
 
-	g_locale_to_utf8 (_("Save thumbnails"), -1, NULL, NULL, NULL);
-	check = gtk_check_button_new_with_label (t);
-	g_free (t);
+	check = gtk_check_button_new_with_label (_("Save thumbnails"));
 	gtk_widget_show (check);
 	gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, TRUE, 0);
 	gtk_tooltips_set_tip (tooltips, check, _("Thumbnails will be "
 			      "saved if this is checked"), NULL);
 	save->priv->toggle_preview = GTK_TOGGLE_BUTTON (check);
 
-	t = g_locale_to_utf8 (_("Save EXIF data"), -1, NULL, NULL, NULL);
-	check = gtk_check_button_new_with_label (t);
-	g_free (t);
+	check = gtk_check_button_new_with_label (_("Save EXIF data"));
 	gtk_widget_show (check);
 	gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, TRUE, 0);
 	gtk_tooltips_set_tip (tooltips, check, _("EXIF data will be "
@@ -548,9 +664,7 @@ gtkam_save_new (void)
 	gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (save)->main_vbox),
 			    hbox, TRUE, TRUE, 0);
 
-	t = g_locale_to_utf8 (_("Open image(s) with: "), -1, NULL, NULL, NULL);
-	label = gtk_label_new (t);
-	g_free (t);
+	label = gtk_label_new (_("Open image(s) with: "));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
@@ -563,10 +677,7 @@ gtkam_save_new (void)
 	save->priv->program = GTK_ENTRY (entry);
 
 	/* Filenames provided by camera */
-	t = g_locale_to_utf8 (_("Use filename(s) provided by the camera"),
-			      -1, NULL, NULL, NULL);
-	check = gtk_check_button_new_with_label (t);
-	g_free (t);
+	check = gtk_check_button_new_with_label (_("Use filename(s) provided by the camera"));
 	gtk_widget_show (check);
 	gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (save)->main_vbox),
 			    check, TRUE, TRUE, 0);
@@ -585,9 +696,7 @@ gtkam_save_new (void)
                                hbox, 256);
 	save->priv->hbox_prefix = hbox;
 
-	t = g_locale_to_utf8 (_("Filename prefix: "), -1, NULL, NULL, NULL);
-        label = gtk_label_new (t);
-	g_free (t);
+        label = gtk_label_new (_("Filename prefix: "));
         gtk_widget_show (label);
         gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
@@ -596,10 +705,7 @@ gtkam_save_new (void)
         gtk_box_pack_start (GTK_BOX (hbox), save->priv->prefix_entry,
 			    TRUE, TRUE, 0);
 
-	t = g_locale_to_utf8 (_("Start numbering with: "),
-			      -1, NULL, NULL, NULL);
-	label = gtk_label_new (t);
-	g_free (t);
+	label = gtk_label_new (_("Start numbering with: "));
         gtk_widget_show (label);
         gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
@@ -610,8 +716,12 @@ gtkam_save_new (void)
         gtk_box_pack_start (GTK_BOX (hbox), save->priv->spin_entry,
 			    FALSE, FALSE, 0);
 
+	load_save_settings(save);
 	gtk_widget_grab_focus (GTK_FILE_SELECTION (save)->ok_button);
 
+	/* Remember the main window (if given) */
+	save->priv->main_window = main_window ? main_window : GTK_WINDOW (save);
+
 	return (GTK_WIDGET (save));
 }
 
